/**
 * @copyright nhcarrigan
 * @license Naomi's Public License
 * @author Naomi Carrigan
 */

import { readFile } from "node:fs/promises";
import { homedir } from "node:os";
import { join } from "node:path";
import { Client } from "discord-rpc";
import type { Config } from "./types.ts";

void (async(): Promise<void> => {
  const configFile = join(homedir(), ".config", "naomis-drpc", "config.json");

  const rawConfig = await readFile(configFile, "utf-8").catch(() => {
    return null;
  });
  if (!rawConfig) {
    process.stderr.write(
      `Config file not found. Please run naomis-drpc-setup first, or manually create a config in ${configFile}.\n`,
    );
    process.exit(1);
  }

  // eslint-disable-next-line @typescript-eslint/consistent-type-assertions -- JSON doesn't take a generic.
  const config = JSON.parse(rawConfig) as Config;
  const {
    appId,
    details,
    state,
    largeImageKey,
    largeImageText,
    smallImageKey,
    smallImageText,
    buttonOneLabel,
    buttonOneUrl,
    buttonTwoLabel,
    buttonTwoUrl,
  } = config;

  if (
    !appId
    || !details
    || !state
    || !largeImageKey
    || !largeImageText
    || !smallImageKey
    || !smallImageText
  ) {
    process.stderr.write(
    // eslint-disable-next-line stylistic/max-len -- This is a long string.
      "Config file is missing required fields. Please run naomis-drpc-setup to reconfigure.\n",
    );
    process.exit(1);
  }

  process.stdout.write("Config loaded successfully.\n");

  const buttons: Array<{ label: string; url: string }> = [];
  if (buttonOneLabel && buttonOneUrl) {
    buttons.push({ label: buttonOneLabel, url: buttonOneUrl });
  }
  if (buttonTwoLabel && buttonTwoUrl) {
    buttons.push({ label: buttonTwoLabel, url: buttonTwoUrl });
  }

  const client = new Client({ transport: "ipc" });

  const startTimestamp = Date.now();

  client.on("ready", () => {
    void client.setActivity({
      buttons,
      details,
      largeImageKey,
      largeImageText,
      smallImageKey,
      smallImageText,
      startTimestamp,
      state,
    });
  });

  await client.login({ clientId: appId });
})();
